/******************************************************************************
*                                                                             *
*   E a s y C h a r                                                           *
*                                                                             *
*   by Stefan Heule                                                           *
*   member of boolsoft (www.boolsoft.org)                                     *
*                                                                             *
*   Source Code                                                               *
*                                                                             *
*   File:   install.c                                                         *
*   Use:    - create charlist                                                 *
*                                                                             *
*******************************************************************************

EasyChar - Replacement of the standard char menu
Copyright (C) 2007  Stefan Heule

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


// ===============================================================
// defines
// ===============================================================
// if you change the backup struct, change this macig, too!
#define BACKUP_FILE_MAGIC "v0.01"

#define BACKUP_SIZE 24*15+6 // 4 tabs with 6 lines with each 15 chars + file magic

// name of the backup file
#define OPTIONS_NAME SYMSTR("charlist")


// our charlist: 6 tabs with each 6 rows and 15 columns
unsigned char char_list[24][15] = {
	// favorites
	{0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 },                // title: favorites
	{28,29,30,31,150,152,176,159,153,154,155,156,157,158,168},     // favorites 1
	{172,185,178,179,180,215,247,177,14,15,16,17,18,19,20},        // favorites 2
	{0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 },                // title: often used
	{0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 },                // often used 1
	{0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 },                // often used 2
	
	// math
	{0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 },                // title: math
	{28,29,30,31,150,152,176,159,153,154,155,156,157,158,168},     // math 1
	{172,185,178,179,180,215,247,177,0 ,0 ,0 ,0 ,0 ,0 ,0 },        // math 2
	{0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 },                // title: greek
	{128,129,130,131,132,133,134,135,136,137,138,139,140,141,142}, // greek 1
	{143,144,145,146,147,148,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 },          // greek 2
	
	// special
	{0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 },                // title: special
	{14,15,16,17,18,19,20,21,22,23,24,25,26,27,127},               // special
	{0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 },                // title: punctuation
	{33,'"','#','$','&','\'','?','@','_','`','|','~',160,161,162}, // punctuation 1
	{163,164,165,166,167,169,174,182,191,171,187,0 ,0 ,0 ,0 },     // punctuation 2
	{0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 ,0 },                // -- unused --
	
	// international
	{192,224,193,225,194,226,195,227,196,228,197,229,198,230,0 },  // A
	{200,232,201,233,202,234,203,235,0 ,0 ,0 ,0 ,0 ,0 ,0 },        // E
	{204,236,205,237,206,238,207,239,0 ,0 ,0 ,0 ,0 ,0 ,0 },        // I
	{210,242,211,243,212,244,213,145,214,246,216,248,186,0 ,0 },   // O
	{217,249,218,250,219,251,220,252,0 ,0 ,0 ,0 ,0 ,0 ,0 },        // U
	{199,231,208,240,209,241,223,222,254,221,253,255,0 ,0 ,0 },    // other
};


//==========================================================================
// includes
//==========================================================================
#include <tigcclib.h>


//==========================================================================
// creates the charlist file with the default content
//==========================================================================
void _main(void)
{
	HANDLE h;
	SYM_ENTRY *symPtr;
	char *base;
	
	// unarchive the variable
	EM_moveSymFromExtMem(OPTIONS_NAME, HS_NULL);
	
	// delete any old backup
	SymDel(OPTIONS_NAME);
	
	// allocate some memory
	if (!(h = HeapAlloc(BACKUP_SIZE + 8)))
	{
		ST_helpMsg("ERROR: Out of mem!");
		return;
	}
	
	if (!(symPtr = DerefSym(SymAdd(OPTIONS_NAME))))
	{
		HeapFree(h);
		ST_helpMsg("ERROR: Failed to create the file");
		return;
	}
	
	*(long*) HeapDeref(symPtr->handle = h) = 0x00010000;
	
	base = HeapDeref(h);
	
	*(unsigned short*)base = BACKUP_SIZE + 6;
	
	// write the extension (cfg)
	*(unsigned char*)(base + BACKUP_SIZE + 2)  = 0;
	memcpy(base + BACKUP_SIZE + 3, "lst", 3);
	*(unsigned char*)(base + BACKUP_SIZE + 6)  = 0;
	*(unsigned char*)(base + BACKUP_SIZE + 7)  = OTH_TAG;
	
	// magic
	memcpy((base + 2), BACKUP_FILE_MAGIC, 6);
	
	// copy the options
	memcpy((base + 2 + 6), char_list, 24*15);
	
	ST_helpMsg("Created the charlist "BACKUP_FILE_MAGIC);
}


































