/******************************************************************************
*                                                                             *
*   E a s y C h a r                                                     			*
*                                                                             *
*   by Stefan Heule																														*
*   member of boolsoft (www.boolsoft.org)																			*
*                                                                             *
*   Source Code			                                                          *
*                                                                             *
*   File:   install.c                                                         *
*   Use:    - source code file                                                *
*           - installation routine of the hook                                *
*                                                                             *
*******************************************************************************

EasyChar - Replacement of the standard char menu
Copyright (C) 2007  Stefan Heule

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

// ===============================================================
// custom events
// ===============================================================
#define CUSTOM_CM_BACKUP_SETTINGS_ALL_TSR   0x6000
#define CUSTOM_CM_RESTORE_SETTINGS_ALL_TSR  0x6001
#define CUSTOM_CM_BACKUP_SETTINGS_AMSEXT    0x6002
#define CUSTOM_CM_RESTORE_SETTINGS_AMSEXT   0x6003
#define CUSTOM_CM_BACKUP_SETTINGS_EASYCHAR  0x6004
#define CUSTOM_CM_RESTORE_SETTINGS_EASYCHAR 0x6005


// ===============================================================
// defines
// ===============================================================
#define _NO_INCLUDE_PATCH

// TSR_NAME: 8 non-zero characters
#define TSR_NAME     "EasyChar"

#define HOOK_NAME    "EasyChar"
#define HOOK_VERSION "1.0.0"

// smaller code
#define OPTIMIZE_ROM_CALLS
#define NO_CALC_DETECT

// produce code for all models
#define USE_TI89
#define USE_TI92
#define USE_V200



// min ams version
#define MIN_AMS 200

// offsets where the table will be displayed
#define OFFSET_Y (TI89 ? 2 : 12)
#define OFFSET_X 1

// if you change the backup struct, change this macig, too!
#define BACKUP_FILE_MAGIC "v0.01"

// size of the backup file
#define BACKUP_SIZE 24*15+6 // 4 tabs with 6 lines with each 15 chars + file magic

// name of the backup file
#define OPTIONS_NAME "charlist"

unsigned char magic[6];
char folder[8+8+1+1];



//==========================================================================
// includes
//==========================================================================
#include <tigcclib.h>



//==========================================================================
// prototypes and globals
//==========================================================================
// trs-routines
void Install_TSR(void);
void EventHandler(EVENT *ev);

// own routines
TEXT_EDIT *TE_findHomeScreen(void);
void DrawDialog(short x0, short y0, short x1, short y1, const char* title);
void DrawRect(short x0, short y0, short x1, short y1, short fill, short color);
void Switch(void);
short getKey(void);
short IsKeyWaiting(void);
void add(short *a, short max);
void sub(short *a, short max);
short createBackup(void);
short loadBackup(void);
short saveBackup(void);

// home entry line
TEXT_EDIT *entryLine;
const char *entryLineText;

// temp vars (for loops, et cetera)
short i,j;

// key input
short key;

// cursor position
short posx;
short posy;

// current tab
short tab;

// backbuffer (NOT full screen size!)
void* screen;

// temp string buffer
char buffer[10];

// handle to get things from the clipboard
HANDLE h;

// length of the clipboard-content
unsigned long len;

// internal clipboard
short clipboard;

// our charlist: 4 tabs with each 6 rows and 15 columns
unsigned char* char_list;


/*
	EV_hook Header
*/
// The Total Length of this header is 22 bytes
// The Offset to jmp Install_TSR:l is 16 bytes
// The Offset to gpOldHandler is 12 bytes
// Here is now the patch for TIGCC 0.94 from Kevin Kovler (added by Thomas Naeff)
asm(".section _stl1 | First library startup section. This is what the executable starts with.
		.ascii \"evHk" TSR_NAME "\"
		.globl gpOldHandler
		gpOldHandler:
		.long 0
		__jmp__Event_Hook__:
		jmp Install_TSR:l

		.global __calculator
		__calculator:
		.word 0");
/*
	End of EV_hook Header
*/


//a pointer to a pointer to the old handler
extern EVENT_HANDLER gpOldHandler;

//==========================================================================
// install function -  this function will be executed ONCE, and ONLY ONCE
//==========================================================================
void Install_TSR(void)
{
	extern char __jmp__Event_Hook__;
	char *ptr=&__jmp__Event_Hook__;
	
	// set EventHandler as the default entry point from now on
	// instead of Install_TSR (this function)
	*(unsigned long*)&ptr[2]=(unsigned long)EventHandler;
	
	#ifndef NO_CALC_DETECT
		asm volatile("
		/* Start Of Calculator Detection Code -- From TIGCC Sources */
				move.l 0xC8,%%a0
				/* Calculator Detection */
				moveq #1,%%d0
				move.l %%a0,%%d1
				and.l #0x400000,%%d1
				jbne __calc_in_d0__
				clr.w %%d0
				move.l (%%a0,0x2F*4),%%a1 /* ScrRect */
				cmp.b #200,2(%%a1)
				jbcs __calc_in_d0__
				moveq #3,%%d0
			__calc_in_d0__:
				lea __calculator(%%pc),%%a1
				move.w %%d0,(%%a1)
		/* End Of Calculator Detection Code -- From TIGCC Sources */"
		:
		:
		: "a0", "a1", "d1");
	#endif
	
	// one time initialisation:
	
	// we start on the first tab on the first entry
	tab = 0;
	posx = posy = 0;
}

//==========================================================================
// processes all necessary events
//==========================================================================
void EventHandler(EVENT *ev)
{
	// Prepatory Setup for the Event Handler
	
	// a special variable: pointer to a function
	EVENT_HANDLER OldHook;
	
	#ifdef OPTIMIZE_ROM_CALLS
		void *saved_a5; // a variable to save the register a5 so it can be restored later
		//asm("/*START: Support for OPTIMIZE_ROM_CALLS*/");
		//asm("move.l %%a5,%0":"=p"(__saved_a5__));
		//asm("move.l 0xC8,%a5");
		//asm("/*END: Support for OPTIMIZE_ROM_CALLS*/");
		saved_a5 = __jmp_tbl;
		__jmp_tbl = *(void**)0xC8;
	#endif
	
	// hook the char menu
	if (ev->Type == CM_KEYPRESS && ev->extra.Key.Code == KEY_CHAR)
	{
		// we don't need this event anymore (maybe later)
		ev->Type = CM_IDLE;
		
		strcpy(folder, "main\\"OPTIONS_NAME);
		
		// allocate some space for our virtual scree (backbuffer)
		if ((screen = malloc(2*95*18*8)) == NULL)
		{
			ST_helpMsg("Out of mem!");
			goto error;
		}
		
		if ((char_list = malloc(24*15)) == NULL)
		{
			free(screen);
			ST_helpMsg("Out of mem!");
			goto error;
		}
		
		// load the char_list
		if (loadBackup() == FALSE)
		{
			goto error;
		}
		
		// if there is one single char in the clipboard, but it into our internal clipboard
		if (CB_fetchTEXT(&h, &len) && len == 1)
		{
			clipboard = *((const char*)HeapDeref(h));
		}
		
		// backup orginal screen
		for (i = 0; i < 95; i++)
		{
			memcpy(screen+95*18*8+i*18, LCD_MEM+(i+OFFSET_Y)*30+OFFSET_X, 18);
		}
		
		// set the graphic port to our virtual screen for backbuffering
		PortSet(screen, 143, 94);
		
		// main loop
		while (1)
		{
			// check if the cursor is at a valid position
			i = 0;
			while (char_list[(tab*6+posy)*15+posx] == 0)
			{
				if (i > 15)
				{
					add(&posy, 5);
					posx = -1;
				}
				i++;
				add(&posx, 14);
			}
			
			// main border and (white) background
			memset(screen, 0, 144*95);
			DrawRect(0, 2, 142, 94, 0, 2);
			
			// titlebar and title
			FontSetSys(F_4x6);
			DrawRect(2, 0, 140, 11, 1, 1);
			DrawStr(3, 4, HOOK_NAME" "HOOK_VERSION, A_XOR);
			DrawPix(1, 1, 1);
			DrawPix(141, 1, 1);
			
			// tabs
			DrawRect(3, 13, 139, 21, 0, 1);
			DrawStr(4, 15, "Favorite   Math   Special   International", A_XOR);
			DrawLine(36, 13, 36, 21, 1);
			DrawLine(59, 13, 59, 21, 1);
			DrawLine(90, 13, 90, 21, 1);
			
			// main tab
			DrawRect(3, 21, 139, 83, 0, 1);
			
			// status line
			DrawRect(3, 83, 139, 91, 0, 1);
			DrawStr(138-DrawStrWidth("F1:Help", F_4x6), 85, "F1:Help", A_XOR);
			if (char_list[(tab*6+posy)*15+posx] != 0)
			{
				sprintf(buffer, "0x%02X, %i", char_list[(tab*6+posy)*15+posx], char_list[(tab*6+posy)*15+posx]);
				DrawStr(4, 85, buffer, A_XOR);
			}
			
			// draw all char's
			FontSetSys(F_6x8);
			for (i = 0; i < 15; i++)
			{
				for (j = 0; j < 6; j++)
				{
					if (char_list[(tab*6+j)*15+i] != 0)
						DrawChar(i*9+5, j*10+24, char_list[(tab*6+j)*15+i], A_XOR);
				}
			}
			
			// draw the selector
			DrawRect(posx*9+5, posy*10+23, posx*9+11, posy*10+31, 1, 3);
			
			// tab specific things
			FontSetSys(F_4x6);
			if (tab == 0)
			{
				DrawLine(4, 21, 35, 21, 0);
				DrawStr(5, 56, "Last used:", 1);
				DrawStr(5, 26, "Favorites:", 1);
			}
			if (tab == 1)
			{
				DrawLine(37, 21, 58, 21, 0);
				
				DrawStr(5, 26, "Math:", 1);
				DrawStr(5, 56, "Greek:", 1);
			}
			if (tab == 2)
			{
				DrawLine(60, 21, 89, 21, 0);
				
				DrawStr(5, 26, "Special:", 1);
				DrawStr(5, 46, "Punctuation:", 1);
			}
			if (tab == 3)
			{
				DrawLine(91, 21, 138, 21, 0);
			}
			
			// copy everything to the screen
			Switch();
			
			// check for a keypress
			key = getKey();
			
			// cursor right
			if (key == KEY_RIGHT)
			{
				add(&posx, 14);
				i = 0;
				while (char_list[(tab*6+posy)*15+posx] == 0)
				{
					if (i > 15)
					{
						add(&posy, 5);
						posx = -1;
					}
					i++;
					add(&posx, 14);
				}
			}
			
			// cursor left
			else if (key == KEY_LEFT)
			{
				sub(&posx, 14);
				i = 0;
				while (char_list[(tab*6+posy)*15+posx] == 0)
				{
					if (i > 15)
					{
						sub(&posy, 5);
						posx = 15;
					}
					i++;
					sub(&posx, 14);
				}
			}
			
			// cursor up
			else if (key == KEY_UP)
			{
				sub(&posy, 5);
				i = 0;
				while (char_list[(tab*6+posy)*15+posx] == 0)
				{
					if (i > 15)
					{
						sub(&posx, 14);
						posy = 6;
					}
					i++;
					sub(&posy, 5);
				}
			}
			
			// cursor down
			else if (key == KEY_DOWN)
			{
				add(&posy, 5);
				i = 0;
				while (char_list[(tab*6+posy)*15+posx] == 0)
				{
					if (i > 6)
					{
						add(&posx, 14);
						posy = -1;
					}
					i++;
					add(&posy, 5);
				}
			}
			
			// pasting the char without leaving
			else if (key == KEY_DIAMOND+KEY_ENTER || key == KEY_ENTER+4096)
			{
				
				// restore the graphic port
				PortRestore();
				
				// restore screen
				for (i = 0; i < 95; i++)
				{
					memcpy(LCD_MEM+(i+OFFSET_Y)*30+OFFSET_X, screen+95*18*8+i*18, 18);
				}
				
				
				// init the event
				EVENT tmp;
				tmp.Type = CM_PASTE_STRING;
				buffer[0] = char_list[(tab*6+posy)*15+posx];
				buffer[1] = 0;
				tmp.extra.pasteText = buffer;
				
				// last used
				for (i = 0; i < 15; i++)
				{
					for (j = 4; j < 6; j++)
					{
						// if the char already was in the often used list, move it to the begining
						if (char_list[j*15+i] == char_list[(tab*6+posy)*15+posx])
						{
							memmove(&char_list[4*15+1], &char_list[4*15+0], (j-4)*15+i);
							char_list[4*15+0] = char_list[(tab*6+posy)*15+posx];
							i = j = 120; // out of both loops
						}
						
						// if no entry is found
						if (j == 5 && i == 14)
						{
							memmove(&char_list[4*15+1], &char_list[4*15+0], 29);
							char_list[4*15+0] = char_list[(tab*6+posy)*15+posx];
						}
					}
				}
				
				// send the event
				EV_sendEvent(AP_CURRENT ,&tmp);
				
				// backup orginal screen
				for (i = 0; i < 95; i++)
				{
					memcpy(screen+95*18*8+i*18, LCD_MEM+(i+OFFSET_Y)*30+OFFSET_X, 18);
				}
				
				// set the graphic port to our virtual screen for backbuffering
				PortSet(screen, 143, 94);
			}
			
			// next tab
			else if (key == KEY_RIGHT+KEY_DIAMOND || key == KEY_APPS)
			{
				add(&tab, 3);
				posx = posy = 0;
			}
			
			// previous tab
			else if (key == KEY_LEFT+KEY_DIAMOND)
			{
				sub(&tab, 3);
				posx = posy = 0;
			}
			
			// exit
			else if (key == KEY_ESC) {
				break;
			}
			
			// help
			else if (key == KEY_F1)
			{
				// white background
				DrawRect(4, 14, 138, 90, 1, 0);
				
				//title
				DrawStr(124, 4, "Help", A_XOR);
				
				DrawStr(6, 17+0*8, "Enter", A_XOR);
				DrawStr(6, 17+1*8, "\x7F"" + ENTER", A_XOR);
				DrawStr(6, 17+2*8, "\x7F"" + \x11/\x12", A_XOR);
				DrawStr(6, 17+3*8, "APPS", A_XOR);
				DrawStr(6, 17+4*8, "2nd+arrows", A_XOR);
				DrawStr(6, 17+5*8, "\x7F"" + C", A_XOR);
				DrawStr(6, 17+6*8, "\x7F"" + V", A_XOR);
				DrawStr(6, 17+7*8, "F2", A_XOR);
				DrawStr(6, 17+8*8, "F3", A_XOR);
				
				DrawStr(50, 17+0*8, "Paste current char", A_XOR);
				DrawStr(50, 17+1*8, "Paste char, no leaving", A_XOR);
				DrawStr(50, 17+2*8, "Switch between tabs", A_XOR);
				DrawStr(50, 17+3*8, "Switch between tabs", A_XOR);
				DrawStr(50, 17+4*8, "Jumt to end/beginning", A_XOR);
				DrawStr(50, 17+5*8, "Copy selected char", A_XOR);
				DrawStr(50, 17+6*8, "Replace selected char", A_XOR);
				DrawStr(50, 17+7*8, "Open standard charmenu", A_XOR);
				DrawStr(50, 17+8*8, "Open 'Set new char'", A_XOR);
				
				// copy everything to the screen
				Switch();
				
				getKey();
			}
			
			// old menu
			else if (key == KEY_F2)
			{
				ev->Type = CM_KEYPRESS;
				
				break;
			}
			
			// new char
			else if (key == KEY_F3)
			{
				short char_num = 0;
				short cursor = 0;
				
				// set buffer to 0
				memset(buffer, 0, 4);
				
				do
				{
					// draw everything
					
					// draw dialog
					DrawDialog(27, 29, 107, 64, "Find char");
					
					// input field
					DrawRect(85, 40, 102, 48, 0, 1);
					
					// draw string
					FontSetSys(F_4x6);
					DrawStr(87, 42, buffer, A_XOR);
					
					// string "enter number"
					DrawStr(31, 42, "Enter number:", A_XOR);
					
					// string "preview"
					DrawStr(31, 53, "Preview:", A_XOR);
					
					// draw preview
					FontSetSys(F_6x8);
					if (atoi(buffer) > 0)
						DrawChar(95, 52, atoi(buffer), 1);
					
					// cursor
					strcpy(&buffer[5], buffer);
					buffer[5+cursor] = 0;
					j = DrawStrWidth(&buffer[5], F_4x6);
					DrawLine(87+j, 41, 87+j, 47, 1);
					
					Switch();
					
					// get a input
					key = getKey();
					
					if ((key >= '0' && key <= '9') && char_num < 3)
					{
						for (i = char_num; i > cursor; i--)
							buffer[i] = buffer[i-1];
						buffer[cursor] = key;
						char_num++;
						buffer[char_num] = 0;
						cursor++;
						
						// check for limit
						if (atoi(buffer) > 255)
							sprintf(buffer, "%i", 255);
					}
					
					else if ((key == KEY_BACKSPACE) && (char_num > 0) && (cursor > 0))
					{
						for (i = cursor-1; i < char_num; i++)
							buffer[i] = buffer[i+1];
						char_num--;
						buffer[char_num] = 0;
						cursor--;
					}
					
					else if (key == KEY_ENTER)
					{
						i = atoi(buffer);
						break;
					}
					else if (key == KEY_ESC)
					{
						i = 0;
						break;
					}
					else if (key == KEY_RIGHT && cursor < char_num)
					{
						cursor++;
					}
					else if (key == KEY_LEFT && cursor > 0)
					{
						cursor--;
					}
					else if (key == KEY_UP)
					{
						strcpy(&buffer[5], buffer);
						buffer[5+strlen(buffer)] = 0;
						i = atoi(&buffer[5]);
						if (i == 255) i--;
						sprintf(buffer, "%i", i+1);
						cursor = strlen(buffer);
					}
					else if (key == KEY_DOWN)
					{
						strcpy(&buffer[5], buffer);
						buffer[5+strlen(buffer)] = 0;
						i = atoi(&buffer[5]);
						if (i == 0) i++;
						sprintf(buffer, "%i", i-1);
						cursor = strlen(buffer);
					}
				} while (TRUE);
				
				// set new char
				if (i)
				{
					if (posy > 0 && posy < 3 && tab == 0)
					{
						char_list[(tab*6+posy)*15+posx] = i;
					}
					else
					{
						clipboard = i;
						ST_helpMsg("Can't replace, put char into clipboard!");
					}
				}
				
				key = 0;
			}
			
			// store the char in the internal clipboard
			else if (key == KEY_DIAMOND+'C')
			{
				clipboard = char_list[(tab*6+posy)*15+posx];
			}
			
			// replace the actual char (if it is a favorite)
			else if (key == KEY_DIAMOND+'V' && clipboard)
			{
				if (posy > 0 && posy < 3 && tab == 0)
				{
					char_list[(tab*6+posy)*15+posx] = clipboard;
				}
				else
				{
					ST_helpMsg("Can't replace this char!");
				}
			}
			
			// end of line
			else if (key == 4096+KEY_RIGHT)
			{
				while (char_list[(tab*6+posy)*15+posx+1] && posx < 14) {
					posx++;
				}
			}
			
			// beginning of line
			else if (key == 4096+KEY_LEFT)
			{
				while (char_list[(tab*6+posy)*15+posx-1] && posx > 0) {
					posx--;
				}
			}
			
			// end of column
			else if (key == 4096+KEY_DOWN)
			{
				for (i = 5; i > posy; i--) {
					if (char_list[(tab*6+i)*15+posx]) {
						posy = i;
						break;
					}
				}
			}
			
			// beginning of column
			else if (key == 4096+KEY_UP)
			{
				for (i = 0; i < posy; i++) {
					if (char_list[(tab*6+i)*15+posx]) {
						posy = i;
						break;
					}
				}
			}
			
			// paste the character
			else if (key == KEY_ENTER)
			{
				// often used
				for (i = 0; i < 15; i++)
				{
					for (j = 4; j < 6; j++)
					{
						// if the char already was in the often used list, move it to the begining
						if (char_list[j*15+i] == char_list[(tab*6+posy)*15+posx])
						{
							memmove(&char_list[4*15+1], &char_list[4*15+0], (j-4)*15+i);
							char_list[4*15+0] = char_list[(tab*6+posy)*15+posx];
							i = j = 120; // out of both loops
						}
						
						// if no entry is found
						if (j == 5 && i == 14)
						{
							memmove(&char_list[4*15+1], &char_list[4*15+0], 29);
							char_list[4*15+0] = char_list[(tab*6+posy)*15+posx];
						}
					}
				}
				
				if (clipboard)
				{
					buffer[0] = clipboard;
					buffer[1] = 0;
					CB_replaceTEXT(buffer, 1, 0);
				}
				
				// set the event to <paste>
				ev->Type = CM_PASTE_STRING;
				buffer[0] = char_list[(tab*6+posy)*15+posx];
				buffer[1] = 0;
				ev->extra.pasteText = buffer;
				break;
			}
		}
		
		// store the internal clipboard in the TIOS clipboard
		if (clipboard && key != KEY_ENTER)
		{
			buffer[0] = clipboard;
			buffer[1] = 0;
			CB_replaceTEXT(buffer, 1, 0);
		}
		
		// restore the graphic port
		PortRestore();
		
		// restore screen
		for (i = 0; i < 95; i++)
		{
			memcpy(LCD_MEM+(i+OFFSET_Y)*30+OFFSET_X, screen+95*18*8+i*18, 18);
		}
		
		// free the screen buffer
		free(screen);
		free(char_list);
		
		// erase the status line
		ST_eraseHelp();
		
		if (saveBackup() == FALSE)
		{
			ST_helpMsg("Failed to save the charlist!");
		}
		
		error:
	}
	
	// call the old hook, if there was one
	// get the address of the next TSR in line to be executed NULL if there is none
  OldHook = gpOldHandler;
	
	// call the old hook if there is one to be called
  if(OldHook)
  {
  	//asm("								/*This inline asm takes the place of OldHook(ev);
  	//											the difference is that a2 will also contain ev
  	//											as a parameter to support TEOS. Without this code,
  	//											you would need to use the switch -ffixed a2 when
  	//											compiling this example.*/
  	//move.l %0,%%a2			/*TEOS expects the parameter to be in a2*/
  	//move.l %%a2,-(%%sp)	/*other event hooks expect it on the stack*/
  	//move.l %1,%%a0			/*Get the address to call*/
  	//jsr (%%a0)					/*Call the next event hook*/
  	//addq.l #4,%%sp			/*restore the stack*/
  	//"::"p"(ev),"g"(OldHook):"a0","a2");
  	OldHook(ev);
  }
	
	#ifdef OPTIMIZE_ROM_CALLS
		//asm("/*START: Support for OPTIMIZE_ROM_CALLS*/");
		//asm("move.l %0, %%a5	/*support for OPTIMIZE_ROM_CALLS*/"::"p"(__saved_a5__));
		//asm("/*END: Support for OPTIMIZE_ROM_CALLS*/");
		__jmp_tbl = saved_a5;
	#endif
}

//==========================================================================
// saves the type of <var> as a string stored in <buffer>. Therefore
// <buffer> should be at least 5 bytes wide (3-4 chars + '\0')
//
// this function is assuming that <var> has the type SDT_OTH!!
//==========================================================================
char* VarTypeToStr(SYM_ENTRY *var)
{
	char *tag;
	
	tag = HToESI(var->handle);
	
	tag -= 5;
	
	// 3 letters?
	if (!isalpha(tag[0]))
		tag++;
	
	strncpy(buffer, tag, 4);
	
	buffer[4] = 0;
	
	return buffer;
}


//==========================================================================
// searches the first file called <filename> in all the folders of the calc,
// but it only returns files with the extension "lst"
//==========================================================================
SYM_ENTRY *FindFile(const char *filename, char *folder_name)
{
	SYM_ENTRY *SymPtr;
	
	// scan all the folders and all the files
	SymPtr = SymFindFirst(NULL, FO_RECURSE);
	for ( ;SymPtr != NULL; SymPtr = SymFindNext())
	{
		// its a folder, so store its name, and continue searching
		if (SymPtr->flags.bits.folder == 1)
		{
			if (folder_name)
				strcpy(folder_name, SymPtr->name); // store its name
			continue;
		}
		
		// it's a file, so compare the name
		if (strcmp(SymPtr->name, filename)) // not the right name, continue searching
			continue;
		
		// no custom extention
		if (GetDataType(HToESI(SymPtr->handle)) != SDT_OTH)
			continue;
		
		// extention must be "lst"
		if (!strcmp(VarTypeToStr(SymPtr), "lst"))
			break; // we found it!
	}
	
	return SymPtr;
}


//==========================================================================
// loads the backup
//==========================================================================
short loadBackup(void)
{
	HANDLE h;
	SYM_ENTRY *symPtr;
	char *base;
	
	// is there already a file
	if (!(symPtr = FindFile(OPTIONS_NAME, folder)))
	{
		ST_helpMsg("No charlist found!");
		return FALSE;
	}
	else
	{
		strcat(folder, "\\");
		strcat(folder, OPTIONS_NAME);
		
		// unarchive the var
		EM_moveSymFromExtMem(SYMSTR(folder), HS_NULL);
	}
	
	// lock the handle
	symPtr = DerefSym(SymFind(SYMSTR(folder)));
	HeapLock(h = symPtr->handle);
	base = HeapDeref(h);
	
	// old version?
	if (strncmp((base + 2), BACKUP_FILE_MAGIC, 6) != 0)
	{
		ST_helpMsg("Outdated charlist!");
		return FALSE;
	}
	
	// read the backup
	memcpy(char_list, (base + 2 + 6), 24*15);
	
	HeapUnlock(h);
	
	// archive the file again
	EM_moveSymToExtMem(SYMSTR(folder), HS_NULL);
	
	return TRUE;
}

//==========================================================================
// saves the backup
//==========================================================================
short saveBackup(void)
{
	HANDLE h;
	SYM_ENTRY *symPtr;
	char *base;
	
	// unarchive the variable
	EM_moveSymFromExtMem(SYMSTR(folder), HS_NULL);
	
	// delete any old backup
	SymDel(SYMSTR(folder));
	
	// create a new backup file
	if (!createBackup())
	{
		ST_helpMsg("Failed to create new file!");
		return FALSE;
	}
	
	symPtr = DerefSym(SymFind(SYMSTR(folder)));
	HeapLock(h = symPtr->handle);
	base = HeapDeref(h);
	
	// magic
	memcpy((base + 2), BACKUP_FILE_MAGIC, 6);
	
	// copy the options
	memcpy((base + 2 + 6), char_list, 24*15);
	
	// unlock the handle
	HeapUnlock(h);
	
	// archive the file
	EM_moveSymToExtMem(SYMSTR(folder), HS_NULL);
	
	return TRUE;
}

//==========================================================================
// creates a new backup file
//==========================================================================
short createBackup(void)
{
	HANDLE h;
	SYM_ENTRY *symPtr;
	char *base;
	
	// allocate some memory
	if (!(h = HeapAlloc(BACKUP_SIZE + 8)))
	{
		return FALSE;
	}
	
	if (!(symPtr = DerefSym(SymAdd(SYMSTR(folder)))))
	{
		HeapFree(h);
		return FALSE;
	}
	
	*(long*) HeapDeref(symPtr->handle = h) = 0x00010000;
	
	base = HeapDeref(h);
	
	*(unsigned short*)base = BACKUP_SIZE + 6;
	
	// write the extension (cfg)
	*(unsigned char*)(base + BACKUP_SIZE + 2)  = 0;
	memcpy(base + BACKUP_SIZE + 3, "lst", 3);
	*(unsigned char*)(base + BACKUP_SIZE + 6)  = 0;
	*(unsigned char*)(base + BACKUP_SIZE + 7)  = OTH_TAG;
	
	return TRUE;
}


//==========================================================================
// copies the content of the backbuffer <screen> onto the real screen
//==========================================================================
void Switch(void)
{
	for (i = 0; i < 95; i++)
	{
		memcpy(LCD_MEM+(i+OFFSET_Y)*30+OFFSET_X, screen+i*18, 18);
	}
}

//==========================================================================
// draws a dialog with the coordinates <x0/y0> and <x1/y1> and the
// title <title>
//==========================================================================
void DrawDialog(short x0, short y0, short x1, short y1, const char* title)
{
	// background (white)
	DrawRect(x0+1, y0+7, x1-1, y1-1, 1, 0);
	
	// titelbar
	DrawRect(x0, y0+2, x1, y0+6, 1, 1);
	DrawLine(x0+2,y0 ,x1-2, y0, 1);
	DrawLine(x0+1, y0+1, x1-1, y0+1, 1);
	
	// border arround
	DrawLine(x0+2, y1, x1-2, y1, 1);
	DrawLine(x0, y0+7,x0 ,y1-2, 1);
	DrawLine(x1, y0+7,x1 ,y1-2, 1);
	DrawPix(x0+1, y1-1, 1);
	DrawPix(x1-1, y1-1, 1);
	
	// border inside the window
	DrawRect(x0+2, y0+8, x1-2, y1-2, 0, 1);
	
	// title
	FontSetSys(F_4x6);
	DrawStr((x0+x1-DrawStrWidth(title, F_4x6))/2, y0+1, title, A_XOR);
};

//==========================================================================
// adds 1 to <a>, and if it is bigger than <max>, sets <a> to 0
//==========================================================================
void add(short *a, short max)
{
	(*a)++;
		if (*a > max)
			*a = 0;
}

//==========================================================================
// subtracts 1 from <a>, and if it is negativ, sets <a> to <max>
//==========================================================================
void sub(short *a, short max)
{
	(*a)--;
		if (*a < 0)
			*a = max;
}

//==========================================================================
// checks for keypresses and turns the calc of if APD expired
//==========================================================================
short getKey(void)
{
	void *kbq = kbd_queue();
	unsigned short key;
	
	// restart the APD timer now
	OSTimerRestart(APD_TIMER);
	
	// get an input
	while (OSdequeue(&key, kbq))
	{
		// check if it is necessary to turn the calc off
		if (OSTimerExpired(APD_TIMER))
		{
			off();
			OSTimerRestart(APD_TIMER);
		}
	}
	
	// erase the status line
	ST_eraseHelp();
	
	// allow key repetition
	return key & 0xF7FF;
};

//==========================================================================
// fast replacement for kbhit()
//==========================================================================
short IsKeyWaiting(void)
{
	unsigned short key;
	return OSqinquire(&key,kbd_queue());
};

//==========================================================================
// draws a rect with the coordinates <x0/y0> and <x1/y1>
// if <fill> is 0, the rect will be outlined, else filled
// <color> = 1: normal
// <color> = 2: double border
// <color> = 3: xor
// else:        erase
//==========================================================================
void DrawRect(short x0, short y0, short x1, short y1, short fill, short color)
{
	short att;
	if (color == 1)      att = A_NORMAL;
	else if (color == 2) att = A_NORMAL | B_DOUBLE;
	else if (color == 3) att = A_XOR;
	else                 att = A_REVERSE;
	
	/*if (x1 == -1)
	{
		x1 = LCD_WIDTH/2 + x0/2;
		x0 = (LCD_WIDTH - x0)/2;
	}
	
	if (y1 == -1)
	{
		y1 = LCD_HEIGHT/2 + y0/2;
		y0 = (LCD_HEIGHT - y0)/2;
	}*/
	
	if (fill)
		ScrRectFill(&(SCR_RECT){{x0, y0, x1, y1}}, &(SCR_RECT){{0, 0, 144, 95}}, att);
	else
		DrawClipRect (&(WIN_RECT){x0, y0, x1, y1}, &(SCR_RECT){{0, 0, 144, 95}}, att);
};




















