/******************************************************************************
*                                                                             *
*   E a s y C h a r                                                     			*
*                                                                             *
*   by Stefan Heule																														*
*   member of boolsoft (www.boolsoft.org)																			*
*                                                                             *
*   Source Code			                                                          *
*                                                                             *
*   File:   install.c                                                         *
*   Use:    - source code file                                                *
*           - installation routine of the hook                                *
*                                                                             *
*******************************************************************************

EasyChar - Replacement of the standard char menu
Copyright (C) 2007  Stefan Heule

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

// ===============================================================
// custom events
// ===============================================================
#define CUSTOM_CM_BACKUP_SETTINGS_ALL_TSR   0x6000
#define CUSTOM_CM_RESTORE_SETTINGS_ALL_TSR  0x6001
#define CUSTOM_CM_BACKUP_SETTINGS_AMSEXT    0x6002
#define CUSTOM_CM_RESTORE_SETTINGS_AMSEXT   0x6003
#define CUSTOM_CM_BACKUP_SETTINGS_EASYCHAR  0x6004
#define CUSTOM_CM_RESTORE_SETTINGS_EASYCHAR 0x6005



// ===============================================================
// defines
// ===============================================================
// TSR_NAME: 8 non-zero characters, which will be displayed, when you
// uninstall it.
#define TSR_NAME     "EasyChar"
#define HOOK_NAME    "EasyChar"
#define HOOK_VERSION "1.0.0"

#define EVHK_TAG_HEXA (long)0x6576486B // = 'e' 'v' 'H' 'k'

// no need for exit or atexit
#define NO_EXIT_SUPPORT

#define NO_AMS_CHECK

// smaller code
#define OPTIMIZE_ROM_CALLS
#define NO_CALC_DETECT

// produce code for all models
#define USE_TI92PLUS
#define USE_V200
#define USE_TI89

#define MIN_AMS 200

#define fatal(s) {ST_showHelp(s);return;}


// ===============================================================
// includes
// ===============================================================
#include <tigcclib.h>			// include all header files
#include "h220xTSR.h"		  // Kevin Kofler's hw2 ams2 tsr support
#include "array.h"		  	// the pre-compiled EVENT HOOK


// main function
void _main(void)
{
	volatile HANDLE tsrHandle; 		 // handle of allocated mem for the Hook
	char		*pTsr;	 						 	 // a pointer to the actual mem for the Hook
	unsigned short size = 0;			 // size of the tsr
	ESI argptr;                    // pointer to the arguments
	EVENT ev;                      // event
	const char* arg;               // temp argument storage
	
	// choose the correct hook
	size = sizeof(tsr);
	
	// init the argument pointer
	InitArgPtr(argptr);
	
	// we are just looking for string arguments
	if (GetArgType(argptr) == STR_TAG)
	{
		arg = GetStrnArg(argptr);
		
		if (strchr(arg, 'b'))
		{
			ev.Type = CUSTOM_CM_BACKUP_SETTINGS_EASYCHAR;
			EV_sendEvent(TIOS_EV_getAppID("TIHOME"), &ev);
		}
		
		else if (strchr(arg, 'r'))
		{
			ev.Type = CUSTOM_CM_RESTORE_SETTINGS_EASYCHAR;
			EV_sendEvent(TIOS_EV_getAppID("TIHOME"), &ev);
		}
		
		return;
	}
	
	// check wheter the hook is already installed (orgnially written by Olivier Armand (ExtendeD))
	if(EV_hook)
	{
		char *HookPtr = (char*)EV_hook;
		char *PrevHookPtr = NULL; // points to the previous hook in the linked list
		
		while (HookPtr != NULL)
		{
			HookPtr -= 16; // point to the header of the hook
			
			// if the hook isn't following Kevin Kofler's convention,
			// our hook can't be further in the linked list
			if(*(long*)HookPtr != EVHK_TAG_HEXA)
				break;
			
			// if this is our hook, uninstall it :(
			if(!memcmp(HookPtr, "evHk"TSR_NAME, 12))
			{
				// if our hook is not the first in the linked list
				if (PrevHookPtr
					// Remove our hook from the linked list :
					// the next hook of PrevHookPtr becomes the next hook of HookPtr...
					// but only if the next hook follows Kevin's convention !
					&& (*(long*)(*(char**)(HookPtr+12) - 16) == EVHK_TAG_HEXA))
						*(char**)(PrevHookPtr+12) = *(char**)(HookPtr+12);
				else
					// Our hook is the first in the linked list :
					// EV_hook becomes the hook after our hook in the list
					EV_hook = (EVENT_HANDLER)*(char**)(HookPtr+12);
				// Free the block where the TSR was.
				// (don't forget to make HookPtr leave the ghost space)
				HeapFree (HeapPtrToHandle ((void*)((long)HookPtr ^ (HW_VERSION == 2 ? 0x40000 : 0))));
				fatal (HOOK_NAME" "HOOK_VERSION" uninstalled.");
			}
			
			// HookPtr is now the previous hook pointer
			PrevHookPtr = HookPtr;
			
			// get the address of the entry point of the next hook
			HookPtr = *(char**)(HookPtr+12);
		}
	}
	
	// sort of like a call to a permanent version of enter_ghost_space();
	if(!h220xTSR())
		fatal("Low RAM!");
	
	// allocate RAM for the Event Hook
	if(!(tsrHandle = HeapAllocHigh(size)))
		fatal("Low RAM!");
	
	// get a pointer to the actual mem and make it point in the "ghost space"
	// ghost space is essentially a shadow of regular ram which is not protected very
	// well by the hardware... subtract 0x40000 to get the real address
	pTsr = HeapDeref(tsrHandle)+(HW_VERSION == 2 ? 0x40000 : 0);
	
	// copy the tsr array to its new location
	memcpy(pTsr,tsr,size);
	
	// save the old tsr hook's address for the uninstall prog and for this hook to call
	*(unsigned long*)&pTsr[12] = (unsigned long)EV_hook;
	
	EX_patch(pTsr,pTsr+size);
	ASM_fastcall(&pTsr[16]);
	EV_hook=(EVENT_HANDLER)&pTsr[16];
	
	// last message
	ST_helpMsg(HOOK_NAME" "HOOK_VERSION" installed");
}




















